
"""
Created on Sat Feb 22 07:57:34 2020

"""

import netgen.gui
from netgen.geom2d import unit_square
from ngsolve import *
from math import pi

mesh = Mesh(unit_square.GenerateMesh(maxh=0.3))
order=4

# Viscosity
nu  = 0.1

# exact solution
a=-8*pi*pi/(   1/nu+sqrt(1/(nu*nu)+64*pi*pi) )
exact_u =CoefficientFunction( (  1-exp(a*x)*cos(2*pi*y), a*exp(a*x)*sin(2*pi*y)/(2*pi)  ))

u_1=1-exp(a*x)*cos(2*pi*y)
u_2=a*exp(a*x)*sin(2*pi*y)/(2*pi)
u_1x=-a*exp(a*x)*cos(2*pi*y)
u_1xx=-a*a*exp(a*x)*cos(2*pi*y)
u_1y=2*pi*exp(a*x)*sin(2*pi*y)
u_1yy=4*pi*pi*exp(a*x)*cos(2*pi*y)
u_2x=a*a*exp(a*x)*sin(2*pi*y)/(2*pi)
u_2xx=a*a*a*exp(a*x)*sin(2*pi*y)/(2*pi)
u_2y=a*exp(a*x)*cos(2*pi*y)
u_2yy=-2*pi*a*exp(a*x)*sin(2*pi*y)

exact_p = CoefficientFunction(-0.5*exp(2*a*x)+0.25*0.5*(exp(3*a)-exp(-a))*(1/a) )
p_x=-a*exp(2*a*x)
p_y=0

exact_f_stokes =CoefficientFunction((-nu*(u_1xx+u_1yy) +p_x   ,
                            -nu*(u_2xx+u_2yy)  +p_y  ))


exact_f =CoefficientFunction((-nu*(u_1xx+u_1yy) +p_x  +u_1*u_1x+u_2*u_1y ,
                            -nu*(u_2xx+u_2yy)  +p_y  +u_1*u_2x+u_2*u_2y))

#H(div)-conforming finite element
V1 = HDiv ( mesh, order = order, dirichlet=[1,2,3,4] )
Q = L2( mesh, order = order-1)
fes = FESpace([V1,Q], dgjumps=True)




#trail and test functions
u,  p = fes.TrialFunction()
v,  q = fes.TestFunction()




#define junmp,mean,n,h
n = specialcf.normal(mesh.dim)
h = specialcf.mesh_size
jump_u = u-u.Other()
jump_v = v-v.Other()
mean_dudn = 0.5*n * (Grad(u)+Grad(u.Other()))
mean_dvdn = 0.5*n * (Grad(v)+Grad(v.Other()))


# weak form
alpha = 6
a = BilinearForm(fes)
diffusion = (grad(u)*grad(v)  -div(u)*q -div(v)*p )* dx \
    +alpha*order**2/h*jump_u*jump_v * dx(skeleton=True) \
    +(-mean_dudn*jump_v-mean_dvdn*jump_u) * dx(skeleton=True) \
    +alpha*order**2/h*u*v * ds(skeleton=True) \
    +(-n*grad(u)*v-n*grad(v)*u)* ds(skeleton=True)
a += diffusion
a.Assemble()

f_stokes = LinearForm(fes)
f_stokes += SymbolicLFI(exact_f_stokes*v)
f_stokes.Assemble()


f = LinearForm(fes)
f += SymbolicLFI(exact_f*v)
f.Assemble()

#solve intial value
gfu = GridFunction(fes, name="intial-stokes")
gfu.vec.data = a.mat.Inverse() * f_stokes.vec
Draw (gfu)


#computing intial error
err_u =  sqrt (Integrate ( (gfu.components[0]-exact_u)*(gfu.components[0]-exact_u), mesh))
err_p =  sqrt (Integrate ( (gfu.components[1]-exact_p)*(gfu.components[1]-exact_p), mesh))

# print intial error
print ( err_p)
print ( err_u)








Tol_u=1
while Tol_u > 1e-4:
    
   b = gfu.components[0]
   uup = IfPos(b*n, u, u.Other())
   convection =  (grad(u) * b) * v * dx+ b*n*uup*jump_v * dx(skeleton=True)

   acd = BilinearForm(fes)
   acd += diffusion + convection

   acd.Assemble()

   gfu = GridFunction(fes)
   gfu.vec.data = acd.mat.Inverse(freedofs=fes.FreeDofs(),inverse="umfpack") * f.vec
   
   # n times error
   e_n=err_u
   e_p=err_p

   # n+1 times error
   err_u =  sqrt (Integrate ( (gfu.components[0]-exact_u)*(gfu.components[0]-exact_u), mesh))
   err_p =  sqrt (Integrate ( (gfu.components[1]-exact_p)*(gfu.components[1]-exact_p), mesh))
   
    #compute iteration error
    
   Tol_u=(abs(e_n-err_u))/(e_n+err_u)
   Tol_p=(abs(e_p-err_p))/(e_p+err_p)
    
   #print errors
   print ( "Tol_u=", Tol_u)
   print ( "Tol_p=", Tol_p)
   print ( "err_u=", err_u)
   print ( "err_p=", err_p)

   
   Draw (gfu.components[0],mesh,'velocity')































